<?php

/**
 * Plugin kzRewriting
 *
 * Permet la ré-écritures des Urls sans utiliser le module Rewrite d'Apache2.
 * Exploite le code erreur 404 de HTTP.
 *
 * @package	PLX
 * @author	Jean-Pierre Pourrez "bazooka07"
 **/

if (!defined('PLX_ROOT')) {
	die('Bye !');
}

class kzRewriting extends plxPlugin {
	const HOOKS = array(
		'plxMotorConstruct',
		'Index',
	);
	const BEGIN_CODE = '<?php' . PHP_EOL;
	const END_CODE = PHP_EOL . '?>';
	const HTACCESS_FILE = PLX_ROOT . '.htaccess';
	const PATTERN = '~/(?:blog|article\d{1,4}/|static\d{1,3}/|categorie\d{1,3}/|archives/\d{4}(?:/\d{2})?|tag/\w+|page\d+|preview|telechargement|download).*$~';
	const PATTERN_DESACTIVATE = '~# BEGIN ' . __CLASS__ . '\s*\n.*# END ' . __CLASS__ . '\s*~si';

	public function __construct($lang) {
		parent::__construct($lang);

		if(!defined('PLX_ADMIN')) {
			foreach(self::HOOKS as $hook) {
				$this->addHook($hook, $hook);
			}
		} else {
			$hook = 'plxAdminEditConfiguration';
			# $this->addHook($hook, $hook);
		}
	}

	/*
	 * Mise à jour du fichier .htaccess à l'activation du plugin
	 * */
	public function OnActivate() {
		if(!defined('PLX_ADMIN')) {
			return;
		}

		$htaccess = is_writable(self::HTACCESS_FILE) ? file_get_contents(self::HTACCESS_FILE) : '';
		if(!empty($htaccess)) {
			$htaccess = trim(preg_replace('~# BEGIN -- Pluxml.*# END -- Pluxml\s*$~ms', '', $htaccess));
		}
		if(!empty($htaccess)) {
			$htaccess .= PHP_EOL;
		}
		$thisClass = __CLASS__;
		$urlBase = preg_replace('~/core/admin$~', '', dirname($_SERVER['PHP_SELF']));
		$htaccess .= <<< EOT
# BEGIN $thisClass
ErrorDocument 404 $urlBase/index.php
# END $thisClass
EOT;
		$htaccess .= PHP_EOL;
		if (!plxUtils::write($htaccess, self::HTACCESS_FILE)) {
			plxMsg::Error(sprintf(L_WRITE_NOT_ACCESS, basename(self::HTACCESS_FILE)));
		}
	}

	/*
	 * Mise à jour du fichier .htaccess à la désactivation du plugin
	 * */
	public function OnDeActivate() {
		if (!defined('PLX_ADMIN') or !is_writable(self::HTACCESS_FILE)) {
			return;
		}

		$htaccess = trim(preg_replace(self::PATTERN_DESACTIVATE, '', file_get_contents(self::HTACCESS_FILE)));
		if (empty($htaccess)) {
			unlink(self::HTACCESS_FILE);
		} elseif (!file_put_contents(self::HTACCESS_FILE, $htaccess)) {
			plxMsg::Error(sprintf(L_WRITE_NOT_ACCESS, basename(self::HTACCESS_FILE)));
		}

		# On désactive la ré-écriture des URLs
		$content = array(
			'urlrewriting' => '0'
		);
		$plxAdmin = plxAdmin::getInstance();
		$plxAdmin->editConfiguration($plxAdmin->aConf, $content);
	}

	/* ===== HOOKS ============ */

	/*
	 * Si le plugin est actif, on force la ré-écriture
	 * */
	public function plxMotorConstruct() {
		echo self::BEGIN_CODE;
?>
$this->aConf['urlrewriting'] = 1;
<?php
		echo self::END_CODE;
	}

	/*
	 * Analyse de la requête URL
	 * */
	public function Index() {
		if(
				!isset($_SERVER["REDIRECT_STATUS"]) or
				$_SERVER["REDIRECT_STATUS"] != 404 or
				!preg_match(self::PATTERN, $_SERVER['REQUEST_URI'])
		) {
			return;
		}

		echo self::BEGIN_CODE;
?>
if (!empty($plxMotor->aConf['urlrewriting'])) {
	$kzGet = preg_replace('#^' . dirname($_SERVER['PHP_SELF']) . '/#', '', $_SERVER['REQUEST_URI']);
	$plxMotor->get = $kzGet;
}
<?php
		echo self::END_CODE;
	}

	/*
	 * Désactive la ré-écriture de .htaccess par plxAmdin::editConfiguration()
	 * */
	public function plxAdminEditConfiguration() {
		echo self::BEGIN_CODE;
?>
if (isset($content['urlrewriting'])) {
	unset($content['urlrewriting']);
}
<?php
		echo self::END_CODE;
	}
}
